from typing import Optional
from abc import ABC, abstractmethod
from urllib.parse import urlparse
import ipaddress
import asyncio

import httpx
from loguru import logger

from .config import AppConfig, ProxyPlatformConfig


class ProxyProvider(ABC):
    @abstractmethod
    async def get_proxy(self) -> Optional[str]:
        pass


class DebugProxyProvider(ProxyProvider):
    def __init__(self, platform_config: ProxyPlatformConfig):
        self.proxy = platform_config.get("value")

    async def get_proxy(self) -> Optional[str]:
        return self.proxy


class RemoteProxyProvider(ProxyProvider):
    def __init__(self, platform_config: ProxyPlatformConfig):
        self.config = platform_config
        self.get_proxy_link = platform_config.get("get_proxy_link", None)

    async def get_proxy(self) -> Optional[str]:
        if not self.get_proxy_link:
            return None
        try:
            async with httpx.AsyncClient() as client:
                resp = await client.get(self.get_proxy_link, timeout=5)
                return f"http://{resp.text.strip()}"
        except Exception as e:
            logger.error(f"获取{self.config.get('name')}代理失败: {e}")
            return None


class ProxyPool:
    def __init__(self, app_config: AppConfig):
        # 应用配置
        self.app_config = app_config
        # 代理提供者
        self.providers: list[ProxyProvider] = []
        # 代理队列
        self.lock = asyncio.Lock()
        self.proxy_queue: asyncio.Queue = asyncio.Queue(maxsize=1)
        self._stop_event = asyncio.Event()  # 新增停止信号
        self._cooldown_time = 0.5  # 基础冷却时间
        self._full_queue_cooldown = 5  # 队列满时的冷却时间
        self.invalid_proxy_count = 0
        # 预加载任务
        self._preload_task = None
        # 初始化代理提供者
        self._init_providers()

    def _init_providers(self):
        """根据配置初始化代理提供者"""
        proxy_conf = self.app_config.proxy_config
        if not proxy_conf["enable"]:
            return

        # 根据配置创建代理提供者
        if proxy_conf["use"] == "debug_proxy":
            platforms = [
                p
                for p in proxy_conf["proxy_platforms"]
                if p["name"] == proxy_conf["use"]
            ]
            if platforms:
                self.providers.append(DebugProxyProvider(platforms[0]))
        else:
            # 过滤并排序代理平台
            platforms = [
                p
                for p in proxy_conf["proxy_platforms"]
                if p["name"] == proxy_conf["use"]
            ]
            platforms.sort(key=lambda x: x.get("priority", 200))

            for platform in platforms:
                self.providers.append(
                    RemoteProxyProvider(ProxyPlatformConfig(**platform))
                )

        self._start_preload()

    def _start_preload(self):
        """安全启动预加载任务"""
        if self.providers and not self._preload_task:
            self._preload_task = asyncio.create_task(self._preload_worker())

    async def _preload_worker(self):
        """优化后的预加载协程"""
        while not self._stop_event.is_set():
            try:
                # 动态调整等待时间
                wait_time = self._cooldown_time

                # 检查队列状态
                if self.proxy_queue.full():
                    wait_time = self._full_queue_cooldown
                else:
                    # 尝试获取并填充代理
                    async with self.lock:
                        for provider in self.providers:
                            if self._stop_event.is_set():
                                return

                            proxy = await provider.get_proxy()
                            if proxy and self._validate_proxy(proxy):
                                try:
                                    self.proxy_queue.put_nowait(proxy)
                                    wait_time = 0  # 成功获取后立即继续
                                except asyncio.QueueFull:
                                    logger.warning("代理队列已满，暂停填充")
                                    break  # 队列已满时停止当前循环
                            else:
                                self.invalid_proxy_count += 1

                # 等待计算
                if not self._stop_event.is_set():
                    await asyncio.sleep(wait_time)

            except asyncio.CancelledError:
                logger.info("预加载协程被取消")
                break
            except Exception as e:
                logger.error(f"预加载协程异常: {e}")
                await asyncio.sleep(1)  # 防止异常导致死循环

    def _validate_proxy(self, proxy: str) -> bool:
        """验证代理格式是否符合http://ip:port规范"""
        try:
            # 解析URL
            parsed = urlparse(proxy)
            if parsed.scheme not in ("http", "https"):
                logger.error(f"代理协议错误: {proxy}，必须为http或https")
                return False

            # 验证主机格式
            try:
                ipaddress.IPv4Address(parsed.hostname)
            except ipaddress.AddressValueError:
                logger.error(f"无效的IP地址格式: {parsed.hostname}")
                return False

            # 验证端口
            if not parsed.port:
                logger.error(f"代理地址缺少端口号: {proxy}")
                return False

            if not (1 <= parsed.port <= 65535):
                logger.error(f"端口超出有效范围: {parsed.port}")
                return False

            return True

        except Exception as e:
            logger.error(f"代理验证异常: {proxy} - {str(e)}")
            return False

    async def get_next_proxy(self) -> Optional[str]:
        """获取代理（优先从预加载队列获取）"""
        # 尝试从队列获取代理
        try:
            proxy = self.proxy_queue.get_nowait()
            self.current_proxy = proxy
            return proxy
        except asyncio.QueueEmpty:
            pass

        # 实时获取代理作为后备
        async with self.lock:
            for provider in self.providers:
                proxy = await provider.get_proxy()
                if proxy:
                    self.current_proxy = proxy
                    return proxy
        return None

    async def close(self):
        """优雅关闭预加载任务"""
        self._stop_event.set()
        if self._preload_task and not self._preload_task.done():
            try:
                await asyncio.wait_for(self._preload_task, timeout=5.0)
            except asyncio.TimeoutError:
                logger.warning("预加载协程关闭超时，强制取消")
                self._preload_task.cancel()
                await self._preload_task



# 暴露公共 API
from .client import BaseApiClient
from .scheduler import BaseScheduler
from .config import BaseConfig, AppConfig, ProxyPlatformConfig, ConfigManager
from .proxy import ProxyPool

__all__ = [
    "BaseApiClient",
    "BaseScheduler",
    "BaseConfig",
    "AppConfig",
    "ProxyPlatformConfig",
    "ConfigManager",
    "ProxyPool",
]



from abc import ABC, abstractmethod
from typing import Any, TypeVar
import traceback
import time
import json
from datetime import datetime
import sys
import asyncio

from loguru import logger

from .client import BaseApiClient
from .config import AppConfig

T = TypeVar("T", bound=BaseApiClient)  # 限定为 BaseApiClient 或其子类


class BaseScheduler(ABC):
    def __init__(
        self,
        app_config: "AppConfig",
        max_concurrent_requests: int = 10,
    ):
        self.app_config = app_config
        # concurrency settings
        self.max_concurrent_requests = max_concurrent_requests
        self._request_counter = 0
        self._counter_lock = asyncio.Lock()
        self._semaphore = asyncio.Semaphore(max_concurrent_requests)

    async def start(self):
        """主调度入口"""
        await self._wait_for_execution_time()
        await self._run_concurrent_tasks()

    async def _wait_for_execution_time(self):
        """等待预设执行时间"""
        if self.app_config.execute_datetime:
            execute_time = datetime.strptime(
                self.app_config.execute_datetime, "%Y-%m-%d %H:%M:%S"
            )
            now = datetime.now()
            if execute_time > now:
                await asyncio.sleep((execute_time - now).total_seconds())

    async def _run_concurrent_tasks(self):
        """创建并管理并发任务"""
        tasks = [
            asyncio.create_task(self._managed_worker(i))
            for i in range(self.app_config.concurrency)
        ]
        await asyncio.gather(*tasks)
        logger.info("所有任务已完成")

    async def _managed_worker(self, task_id: int):
        """带信号量控制的worker包装器"""
        async with self._semaphore:
            await self.worker(task_id)

    @abstractmethod
    async def worker(self, task_id: int):
        """需要子类实现的工作协程"""
        pass

    async def execute_operation(self, task_id: int, client: T) -> Any:
        """执行核心操作的模板方法"""
        async with self._counter_lock:
            if self._request_counter >= self.app_config.max_requests:
                return None
            self._request_counter += 1
            current_request = self._request_counter

        start_time = time.time()
        try:
            result = await self.perform_action(client)
            return await self.handle_success(
                task_id=task_id,
                result=result,
                current_request=current_request,
                elapsed=time.time() - start_time,
            )
        except Exception as e:
            return await self.handle_failure(
                task_id=task_id,
                exception=e,
                current_request=current_request,
                elapsed=time.time() - start_time,
            )
        finally:
            await asyncio.sleep(self.app_config.request_delay)

    @abstractmethod
    async def perform_action(self, client: T) -> Any:
        """执行具体业务操作的抽象方法"""
        pass

    async def handle_success(
        self, task_id: int, result: Any, current_request: int, elapsed: float
    ) -> dict:
        """成功处理模板方法"""
        formatted_result = self.format_result(result)
        log_message = self.construct_success_log(
            task_id=task_id,
            current_request=current_request,
            elapsed=elapsed,
            result=formatted_result,
        )
        logger.info(log_message)
        return {
            "status": "success",
            "task_id": task_id,
            "request_num": current_request,
            "result": formatted_result,
        }

    async def handle_failure(
        self, task_id: int, exception: Exception, current_request: int, elapsed: float
    ) -> dict:
        """失败处理模板方法"""
        error_info = self.format_exception(exception)
        log_message = self.construct_failure_log(
            task_id=task_id,
            current_request=current_request,
            elapsed=elapsed,
            error=error_info,
        )
        logger.error(log_message)
        return {
            "status": "failure",
            "task_id": task_id,
            "request_num": current_request,
            "error": error_info,
        }

    # 可覆盖的格式化方法
    def format_result(self, result: Any) -> str:
        return json.dumps(result, ensure_ascii=False, separators=(",", ":"))

    def format_exception(self, exception: Exception) -> str:
        # 获取异常堆栈信息
        exc_type, exc_value, exc_traceback = sys.exc_info()
        # 提取最后一级的堆栈帧（实际发生错误的位置）
        tb_last = traceback.extract_tb(exc_traceback)[-1]
        return (
            f"在 {tb_last.name}() "
            f"({tb_last.filename}:{tb_last.lineno}): {str(exception)}"
        )

    # 日志构建方法
    def construct_success_log(
        self, task_id: int, current_request: int, elapsed: float, result: str
    ) -> str:
        return (
            f"Task-{task_id:02d} | Request-{current_request:03d} | "
            f"Elapsed: {elapsed:.2f}s | "
            f"Response: {result}"
        )

    def construct_failure_log(
        self, task_id: int, current_request: int, elapsed: float, error: str
    ) -> str:
        return (
            f"Task-{task_id:02d} | Request-{current_request:03d} | "
            f"Elapsed: {elapsed:.2f}s | "
            f"Error: {error}"
        )



from abc import ABC, abstractmethod
from dataclasses import dataclass, asdict, field
from typing import Type, TypeVar, Optional, Literal
from typing_extensions import TypedDict
from datetime import datetime
import os
import json

T = TypeVar("T", bound="BaseConfig")  # 所有配置类都继承自BaseConfig


class BaseConfig(ABC):
    @classmethod
    def from_dict(cls: Type[T], data: dict) -> T:
        """统一反序列化入口（需子类实现具体逻辑）"""
        raise NotImplementedError

    @abstractmethod
    def to_dict(self) -> dict:
        """将配置转换为字典格式"""
        raise NotImplementedError


class ProxyPlatformConfig(TypedDict):
    name: str
    value: Optional[str]
    get_proxy_link: Optional[str]
    home_page: Optional[str]
    zh_name: Optional[str]
    priority: int


@dataclass
class ProxyConfig:
    enable: bool
    use: Literal["debug_proxy", "yi_dai_li"]
    proxy_platforms: list[ProxyPlatformConfig]


@dataclass
class AppConfig(BaseConfig):
    execute_datetime: Optional[str] = None
    concurrency: int = 1
    max_requests: int = 10
    max_retries: int = 3
    request_delay: float = 0.5
    request_timeout: float = 10
    fake_headers_enabled: bool = True

    # 新增代理配置结构
    proxy_config: ProxyConfig = field(
        default_factory=lambda: {
            "enable": True,
            "use": "debug_proxy",
            "proxy_platforms": [
                {"name": "debug_proxy", "value": "http://127.0.0.1:7890", "priority": 1}
            ],
        }
    )

    # 可选的额外属性（保持向后兼容）
    _extra: dict = field(default_factory=dict)

    def __post_init__(self):
        """初始化后处理逻辑"""
        if not self.execute_datetime:
            self.execute_datetime = datetime.now().strftime("%Y-%m-%d %H:%M:%S")

        # 将旧配置转换为新结构（兼容旧版本配置）
        if "enable_proxy" in self._extra or "proxy_url" in self._extra:
            self.proxy_config["enable"] = self._extra.get("enable_proxy", False)
            self.proxy_config["use"] = (
                "custom" if self.proxy_config["enable"] else "none"
            )
            if self._extra.get("proxy_url"):
                self.proxy_config["proxy_platforms"] = [
                    {
                        "name": "custom",
                        "get_proxy_link": self._extra["proxy_url"],
                        "home_page": "",
                        "zh_name": "自定义代理",
                        "priority": 1,
                    }
                ]

    @classmethod
    def from_dict(cls, data: dict) -> "AppConfig":
        """从字典创建配置实例"""
        filtered_data = {k: v for k, v in data.items() if k in cls.__annotations__}
        extra = {k: v for k, v in data.items() if k not in cls.__annotations__}

        # 处理新旧配置转换
        if "proxy_config" not in filtered_data and (
            "enable_proxy" in data or "proxy_url" in data
        ):
            filtered_data["proxy_config"] = {
                "enable": data.get("enable_proxy", False),
                "use": "custom" if data.get("enable_proxy") else "none",
                "proxy_platforms": [
                    {
                        "name": "custom",
                        "get_proxy_link": data.get("proxy_url", ""),
                        "home_page": "",
                        "zh_name": "自定义代理",
                        "priority": 1,
                    }
                ]
                if data.get("enable_proxy")
                else [],
            }

        config = cls(**filtered_data)
        config._extra = extra
        return config

    def to_dict(self) -> dict:
        """转换为字典格式"""
        return {**asdict(self), **self._extra}

    # @property
    # def execute_timestamp(self) -> float:
    #     """获取执行时间的时间戳"""
    #     if not self.execute_datetime:  # 检查是否为 None 或空字符串
    #         return datetime.now().timestamp()  # 默认使用当前时间
    #     return datetime.strptime(self.execute_datetime, "%Y-%m-%d %H:%M:%S").timestamp()


class ConfigManager:
    @staticmethod
    def save(config: BaseConfig, file_path: str) -> bool:
        """
        保存配置到JSON文件，自动创建目录并处理异常

        Args:
            config: 配置对象
            file_path: 文件保存路径

        Returns:
            bool: 保存是否成功
        """

        # 确保目录存在
        directory = os.path.dirname(file_path)
        if directory:  # 处理文件在根目录的情况（如file_path为"config.json"）
            try:
                os.makedirs(directory, exist_ok=True)
            except OSError as e:
                print(f"无法创建目录 {directory}: {e}")
                return False

        try:
            # 显式指定文件对象类型为 SupportsWrite[str]
            with open(file_path, "w", encoding="utf-8") as f:
                json.dump(
                    config.to_dict(),
                    f,  # type: ignore[arg-type]
                    indent=2,
                    ensure_ascii=False,
                )
            return True
        except IOError as e:
            print(f"文件写入失败 {file_path}: {e}")
        except Exception as e:
            print(f"保存配置时发生未知错误: {e}")

        return False

    @staticmethod
    def load(file_path: str, config_class: Type[T]) -> T:
        """从JSON文件加载配置

        Args:
            file_path: 配置文件路径
            config_class: 要实例化的配置类（如AppConfig/CacheData）

        Returns:
            指定类型的配置对象实例
        """

        if not os.path.exists(file_path):
            return config_class()

        with open(file_path, "r", encoding="utf-8") as f:
            data = json.load(f)

        # 调用类的from_dict方法进行反序列化
        return config_class.from_dict(data)



from abc import ABC, abstractmethod
from typing import Optional, Any, Union
import random
import ssl

import httpx
from loguru import logger
from fake_useragent import UserAgent

from .config import AppConfig
from .proxy import ProxyPool


class BaseApiClient(ABC):
    def __init__(
        self,
        app_config: AppConfig,
        proxy_pool: Optional[ProxyPool] = None,
        *args,
        **kwargs,
    ):
        # 应用配置
        self.app_config = app_config

        # 缓存数据
        # self.cache_data = cache_data

        # 代理配置
        self.proxy: Optional[str] = None

        # 请求客户端
        self._ssl_context = self._create_ssl_context()
        self._async_client: Optional[httpx.AsyncClient] = None

        # 代理池相关配置
        self.current_proxy = None
        self.proxy_pool = proxy_pool if proxy_pool else ProxyPool(app_config)
        self.max_retries = app_config.max_retries
        self.retry_status_codes = {429, 500, 502, 503, 504}
        self.retry_exceptions = (
            httpx.RequestError,
            httpx.ProxyError,
            httpx.ConnectTimeout,
            ConnectionResetError,
        )

        # 模拟UA
        self.fake_headers_enabled = getattr(app_config, "fake_headers_enabled", True)
        self.ua = UserAgent()

        # 日志消息
        self.execute_message = ""
        self.sign_message = ""

        # 其他数据
        self.args = args
        self.kwargs = kwargs

    async def __aenter__(self):
        self._async_client = await self._create_http_client()
        return self

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.proxy_pool:
            await self.proxy_pool.close()
        if self._async_client:
            await self._async_client.aclose()

    def _create_ssl_context(self):
        ctx = ssl.create_default_context()
        ctx.set_ciphers("ALL")
        return ctx

    def process_cookies(self, cookies: Union[str, dict]) -> dict:
        default_cookies = {}
        if isinstance(cookies, str):
            # 解析字符串并合并到默认值
            cookie_dict = {}
            for pair in cookies.split(";"):
                key, value = pair.strip().split("=", 1)
                cookie_dict[key] = value
            default_cookies.update(cookie_dict)
            return default_cookies
        elif isinstance(cookies, dict):
            # 合并字典到默认值
            default_cookies.update(cookies)
            return default_cookies
        return default_cookies

    async def _create_http_client(self) -> httpx.AsyncClient:
        if self.proxy_pool:
            proxy = await self.proxy_pool.get_next_proxy()
            if proxy:
                self.current_proxy = proxy

        # 处理 cookies
        cookies = self.kwargs.get("cookies")
        processed_cookies = self.process_cookies(cookies) if cookies else None

        return httpx.AsyncClient(
            cookies=processed_cookies,
            timeout=self.app_config.request_timeout,
            http2=True,
            verify=self._ssl_context,
            proxy=self.current_proxy,
        )

    def _generate_fake_headers(self) -> dict:
        if not self.fake_headers_enabled:
            return {}
        return {
            "X-Forwarded-For": self._random_ip(),
            "X-Real-IP": self._random_ip(),
            "User-Agent": self.ua.random,
            "Accept-Language": random.choice(["zh-CN,zh;q=0.9", "en-US,en;q=0.5"]),
            "Cache-Control": "no-cache",
            "Pragma": "no-cache",
            # 可扩展其他安全头
        }

    def _random_ip(self) -> str:
        """生成更真实的公网IPv4地址"""
        while True:
            # 生成第一个八位组（1-254）
            octet1 = random.randint(1, 254)

            # 排除特殊地址段
            if octet1 in {10, 127} or 224 <= octet1 <= 255:
                continue

            # 生成第二个八位组
            octet2 = random.randint(0, 255)
            if octet1 == 172:
                # 排除私有地址段 172.16.0.0 - 172.31.255.255
                if 16 <= octet2 <= 31:
                    continue
            elif octet1 == 192:
                # 排除私有地址段 192.168.0.0/16
                if octet2 == 168:
                    continue

            # 生成第三、第四个八位组
            octet3 = random.randint(0, 255)
            octet4 = random.randint(0, 255)

            # 检查是否为链路本地地址（169.254.0.0/16）
            if octet1 == 169 and octet2 == 254:
                continue

            return f"{octet1}.{octet2}.{octet3}.{octet4}"

    async def _refresh_client(self):
        if self._async_client:
            await self._async_client.aclose()
        self._async_client = await self._create_http_client()

    async def _get_http_client(self) -> httpx.AsyncClient:
        if not self._async_client:
            self._async_client = await self._create_http_client()
        return self._async_client

    async def _request_with_retry(self, method: str, url: str, **kwargs) -> Any:
        # 在请求前添加伪装头
        headers = kwargs.get("headers", {}).copy()
        headers.update(self._generate_fake_headers())
        kwargs["headers"] = headers

        retries = 0
        while retries <= self.max_retries:
            try:
                client = await self._get_http_client()
                response = await client.request(method, url, **kwargs)
                if response.status_code in self.retry_status_codes:
                    raise httpx.HTTPStatusError(
                        f"请求失败，HTTP 状态码为 {response.status_code}，该状态码属于需要重试的范围。",
                        response=response,
                        request=response.request,
                    )
                return response
            except self.retry_exceptions as e:
                logger.warning(f"请求失败: {e}, 重试次数: {retries}/{self.max_retries}")
                await self.proxy_pool.get_next_proxy()
                await self._refresh_client()
                retries += 1
                if retries > self.max_retries:
                    logger.error("达到最大重试次数")
                    raise

    @abstractmethod
    async def perform_action(self, action: str, *args, **kwargs) -> Any:
        """统一操作入口，处理不同业务逻辑"""
        pass
